/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

"use strict";
var OW_GoogleMapLocationSearch = function($, google)
{
    return function(fieldName, addressFieldId, restrictedCountry)
    {
        var self = this;

        var geocoder;
        var map;
        var marker;
        var bounds;
        
        var latitudeField;
        var longitudeField;
        var northEastLat;
        var northEastLng;
        var southWestLat;
        var southWestLng;
        var addressField;
        var jsonField;
        
        var country = restrictedCountry;

        var fieldId = addressFieldId;
        this.isValidValue = false;

        this.initialize = function()
        {        
            geocoder = new google.maps.Geocoder();

            $(function() {
                latitudeField = $('input[name="'+fieldName+'[latitude]"]');
                longitudeField = $('input[name="'+fieldName+'[longitude]"]');
                northEastLat = $('input[name="'+fieldName+'[northEastLat]"]');
                northEastLng = $('input[name="'+fieldName+'[northEastLng]"]');
                southWestLat = $('input[name="'+fieldName+'[southWestLat]"]');
                southWestLng = $('input[name="'+fieldName+'[southWestLng]"]');
                addressField = $('input[name="'+fieldName+'[address]"]');
                jsonField = $('input[name="'+fieldName+'[json]"]');

                if ( jsonField.val() )
                {
                    self.isValidValue = true;
                }

                $("#"+fieldId + "_delete_icon").click(function() { 
                    self.deleteValue();
                });

                OW_GoogleMapLocationAutocomplete(fieldId,{
                    delay: 250,

                    focus: function (event, ui) {
                        $(".ui-helper-hidden-accessible").hide();
                        event.preventDefault();
                    },

                    source: function(request, response) {

                        var icon= $('#'+fieldId + '_icon');
                        icon.removeClass('ic_googlemap_pin');
                        icon.addClass('ow_inprogress');

                        var geocoderParams = {
                            'address': request.term
                        };
                        
                        if (country)
                        {
                            geocoderParams.componentRestrictions = {country: country};
                        }

                        geocoder.geocode( geocoderParams, function(results, status) {

                            icon.removeClass('ow_inprogress');
                            icon.addClass('ic_googlemap_pin');

                            response($.map(results, function(item) {
                                return {
                                    label:  item.formatted_address,
                                    latitude: item.geometry.location.lat(),
                                    longitude: item.geometry.location.lng(),
                                    value: item.formatted_address,
                                    result: item
                                }
                            }));
                        })
                    },

                    select: function(event, ui) {
                        
                        self.showDeleteIcon();
                        
                        var location = new google.maps.LatLng(ui.item.latitude, ui.item.longitude);

                        addressField.val(ui.item.value);
                        longitudeField.val(ui.item.longitude);
                        latitudeField.val(ui.item.latitude);
                        northEastLat.val(ui.item.result.geometry.viewport.getNorthEast().lat())
                        northEastLng.val(ui.item.result.geometry.viewport.getNorthEast().lng())
                        southWestLat.val(ui.item.result.geometry.viewport.getSouthWest().lat())
                        southWestLng.val(ui.item.result.geometry.viewport.getSouthWest().lng())

                        jsonField.val( JSON.stringify(ui.item.result).replace('"','\"'));
                        self.isValidValue = true;
                    }
                });
            });
        }
        
        this.deleteValue = function(item)
        {
            self.hideDeleteIcon();

            $('#'+fieldId).val('');
            addressField.val('');
            longitudeField.val('');
            latitudeField.val('');
            northEastLat.val('');
            northEastLng.val('');
            southWestLat.val('');
            southWestLng.val('');

            jsonField.val('');
            self.isValidValue = false;
        }
        
        this.showDeleteIcon = function(item)
        {
            $("#"+fieldId + "_icon").hide();
            $("#"+fieldId + "_delete_icon").css("display","inline");
        }

        this.hideDeleteIcon = function(item)
        {
            $("#"+fieldId + "_icon").show();
            $("#"+fieldId + "_delete_icon").hide();
        }
    }
} (locationJquey, google)