<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class HOTLIST_CLASS_EventHandler
{
    /**
     *
     * @var HOTLIST_BOL_Service 
     */
    private $service;
    
    public function __construct() 
    {
        $this->service = HOTLIST_BOL_Service::getInstance();
    }

    public function getCount( OW_Event $event )
    {
        $count = $this->service->getUserCount();
        $event->setData($count);

        return $count;
    }
    
    public function getListIdList( OW_Event $event )
    {
        $params = $event->getParams();
        $offset = empty($params["offset"]) ? 0 : $params["offset"];
        $count = empty($params["count"]) ? null : $params["count"];
        
        $dtoList = $this->service->getHotList($offset, $count);
        $list = array();
        foreach ( $dtoList as $dto )
        {
            $list[] = $dto->userId;
        }
        
        $event->setData($list);
        
        return $list;
    }
    
    public function addToList( OW_Event $event )
    {
        $params = $event->getParams();
        $userId = $params["userId"];
        
        $data = array(
            "result" => true,
            "message" => null,
            "buyCredits" => false
        );
        
        $available = true;
        
        if ( !isset($params["checkCredits"]) || $params["checkCredits"] )
        {
            if ( !OW::getUser()->isAuthorized("hotlist", "add_to_list") )
            {
                $status = BOL_AuthorizationService::getInstance()->getActionStatus("hotlist", "add_to_list");
                $available = false;
                $data["result"] = false;
                
                if ( $status["status"] == BOL_AuthorizationService::STATUS_PROMOTED )
                {
                    $data["message"] = $status["msg"];
                    $data["buyCredits"] = true;
                }
            }
        }
        
        if ( $available )
        {
            $this->service->addUser($userId);
            BOL_AuthorizationService::getInstance()->trackAction('hotlist', 'add_to_list');
            
            $data["result"] = true;
            $data['message'] = OW::getLanguage()->text('hotlist', 'user_added');
        }
        
        $event->setData($data);
        
        return $data;
    }
    
    public function removeFromList( OW_Event $event )
    {
        $params = $event->getParams();
        $userId = $params["userId"];
        
        $this->service->deleteUser($userId);
        
        $data = array(
            "result" => true,
            "message" => OW::getLanguage()->text('hotlist', 'user_removed')
        );

        $event->setData($data);
        
        return $data;
    }
    
    public function isUserAdded( OW_Event $event )
    {
        $params = $event->getParams();
        $userId = $params["userId"];
        
        $userDto = $this->service->findUserById($userId);
        
        $data = $userDto !== null;
        $event->setData($data);
        
        return $data;
    }

    public function addAuthLabels( BASE_CLASS_EventCollector $event )
    {
        $language = OW::getLanguage();

        $event->add(
            array(
                'hotlist' => array(
                    'label' => $language->text('hotlist', 'auth_group_label'),
                    'actions' => array(
                        'add_to_list'=>$language->text('hotlist', 'auth_action_label_add_to_list')
                    )
                )
            )
        );
    }
    
    public function genericInit()
    {
        OW::getEventManager()->bind("hotlist.count", array($this, "getCount"));
        OW::getEventManager()->bind("hotlist.get_id_list", array($this, "getListIdList"));
        OW::getEventManager()->bind("hotlist.add_to_list", array($this, "addToList"));
        OW::getEventManager()->bind("hotlist.remove_from_list", array($this, "removeFromList"));
        OW::getEventManager()->bind("hotlist.is_user_added", array($this, "isUserAdded"));
        OW::getEventManager()->bind('admin.add_auth_labels', array($this, 'addAuthLabels'));
    }
    
    public function init()
    {
        
    }
}