<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class OXDATE_BOL_AccountTypeToGenderDao extends OW_BaseDao
{
    const ACCOUNT_TYPE = 'accountType';
    const GENDER_VALUE = 'genderValue';

    /**
     * Singleton instance.
     *
     * @var BOL_AttachmentDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return BOL_AttachmentDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    protected function __construct()
    {
        parent::__construct();
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'OXDATE_BOL_AccountTypeToGender';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'oxdate_account_type_to_gender';
    }

    public function findByAccountType( $accountType )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::ACCOUNT_TYPE, $accountType);

        return $this->findListByExample($example);
    }

    public function findByGenderValue( $gender )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::GENDER_VALUE, $gender);

        return $this->findListByExample($example);
    }

    public function deleteByAccountType( $accountType )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::ACCOUNT_TYPE, $accountType);

        return $this->deleteByExample($example);
    }

    public function deleteByGenderValue( $gender )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::GENDER_VALUE, $gender);

        return $this->deleteByExample($example);
    }

    public function findByGenderAndAccountType( $accountType, $gender )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::ACCOUNT_TYPE, $accountType);
        $example->andFieldEqual(self::GENDER_VALUE, $gender);

        return $this->findObjectByExample($example);
    }

    public function deleteByAccountTypeList( $accountTypeList )
    {
        if ( empty($accountTypeList) || !is_array($accountTypeList) )
        {
            return;
        }

        $example = new OW_Example();
        $example->andFieldInArray(self::ACCOUNT_TYPE, $accountTypeList);

        $this->deleteByExample($example);
    }

    public function deleteByGenderList( $genderList )
    {
        if ( empty($genderList) || !is_array($genderList) )
        {
            return;
        }

        $example = new OW_Example();
        $example->andFieldInArray(self::GENDER_VALUE, $genderList);

        $this->deleteByExample($example);
    }

    public function batchReplace( array $dtoList )
    {
        $this->dbo->batchInsertOrUpdateObjectList($this->getTableName(), $dtoList);
    }
}
