<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class OXDATE_CMP_UserList extends OW_Component
{
    private $userList = array();
    private $fieldList = array();
    private $displayActivity = TRUE;

    public function __construct( $userList, $fieldList, $usersCount )
    {
        parent::__construct();

        $this->userList = $userList;
        $this->fieldList = array_unique($fieldList);

        $page = (!empty($_GET['page']) && intval($_GET['page']) > 0 ) ? $_GET['page'] : 1;
        $usersOnPage = OW::getConfig()->getValue('base', 'users_on_page');
        $this->addComponent('paging', new BASE_CMP_Paging($page, ceil($usersCount / $usersOnPage), 5));
    }

    public function onBeforeRender()
    {
        parent::onBeforeRender();

        $userList = array();
        $userDtoList = array();

        $userService = BOL_UserService::getInstance();
        $questionService = BOL_QuestionService::getInstance();

        $userIdList = array_keys($this->userList);
        $userDataList = BOL_QuestionService::getInstance()->getQuestionData($userIdList, $this->fieldList);

        foreach ( $userService->findUserListByIdList($userIdList) as $userDto )
        {
            $userDtoList[$userDto->id] = $userDto;
        }

        foreach ( $this->userList as $userId => $fieldList )
        {
            $fields = array_diff(array_keys($fieldList), $this->fieldList);
            $fieldsData = $questionService->getQuestionData(array($userId), $fields);
            $userList[$userId]['fields'] = array_merge(!empty($userDataList[$userId]) ? $userDataList[$userId] : array(), !empty($fieldsData[$userId]) ? $fieldsData[$userId] : array(), $fieldList);
            $userList[$userId]['dto'] = $userDtoList[$userId];
        }

        $this->assign('userList', $userList);
        $this->assign('avatars', BOL_AvatarService::getInstance()->getAvatarsUrlList($userIdList, 2));
        $this->assign('onlineList', !empty($userIdList) ? $userService->findOnlineStatusForUserList($userIdList) : array());
        $this->assign('usernameList', $userService->getUserNamesForList($userIdList));
        $this->assign('displaynameList', $userService->getDisplayNamesForList($userIdList));
        $this->assign('displayActivity', $this->displayActivity);
    }

    public function setDisplayActivity( $value )
    {
        $this->displayActivity = (booL) $value;

        return $this;
    }
}
